'''
Created on 2022-03-06

@author: wf
'''

class Version(object):
    '''
    Version handling for ConferenceCorpus
    '''
    name="pyLodStorage"
    version='0.1.20'
    date = '2020-09-10'
    updated = '2022-03-14'

        